package com.jz.dmp.modules.service.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.common.bean.BaseBeanResponse;
import com.jz.common.bean.BaseResponse;
import com.jz.common.bean.PageInfoResponse;
import com.jz.common.constant.StatuConstant;
import com.jz.common.persistence.BaseService;
import com.jz.common.utils.web.SessionUtils;
import com.jz.dmp.azkaban.dao.ProjectFlowsMapper;
import com.jz.dmp.modules.controller.bean.DmpProjectDto;
import com.jz.dmp.modules.controller.bean.DmpProjectRequest;
import com.jz.dmp.modules.controller.bean.IndexProjAzkabanStatuDto;
import com.jz.dmp.modules.controller.bean.MyDmpProjectConverter;
import com.jz.dmp.modules.dao.DmpProjectDao;
import com.jz.dmp.modules.model.DmpMember;
import com.jz.dmp.modules.model.DmpProject;
import com.jz.dmp.modules.model.DmpProjectConfigInfo;
import com.jz.dmp.modules.model.DmpProjectSystemInfo;
import com.jz.dmp.modules.service.DmpProjectService;
import com.jz.dmp.modules.service.projconfig.DmpProjectConfigInfoService;

/**
 * 项目信息服务的实现?
 * @author ybz
 *
 */
@Service("dmpProjectService")
public class DmpProjectServiceImpl extends BaseService implements DmpProjectService {
	
	private static Logger logger = LoggerFactory.getLogger(DmpProjectServiceImpl.class);
	
	@Autowired
	private DmpProjectDao dmpProjectDao;
	
	@Autowired
	private ProjectFlowsMapper projectFlowsMapper;
	
	@Autowired
	private DmpProjectConfigInfoService dmpProjectConfigInfoService;
	
	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpProjectService#findList(com.ycxc.vmts.controller.bean.DmpProjectRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public PageInfoResponse<DmpProjectDto> findListWithPage(DmpProjectRequest dmpProjectRequest, HttpServletRequest httpRequest)
			throws Exception {
		PageInfoResponse<DmpProjectDto> pageInfoResponse = new PageInfoResponse<>();
		
		Map<String, Object> param = new HashMap<>();
		//ID
		if (dmpProjectRequest.getId() != null) {
			param.put("id", dmpProjectRequest.getId());
		}
		//管理员帐号
		if (!StringUtils.isEmpty(dmpProjectRequest.getOwnerId())) {
			param.put("ownerId", dmpProjectRequest.getOwnerId());
		}
		//生产帐号
		if (!StringUtils.isEmpty(dmpProjectRequest.getProductionId())) {
			param.put("productionId", dmpProjectRequest.getProductionId());
		}
		//数据仓库帐号
		if (!StringUtils.isEmpty(dmpProjectRequest.getDataWarehouseId())) {
			param.put("dataWarehouseId", dmpProjectRequest.getDataWarehouseId());
		}
		//项目名称
		if (!StringUtils.isEmpty(dmpProjectRequest.getName())) {
			param.put("name", dmpProjectRequest.getName());
		}
		//项目显示名称
		if (!StringUtils.isEmpty(dmpProjectRequest.getDisplayName())) {
			param.put("displayName", dmpProjectRequest.getDisplayName());
		}
		//项目描述
		if (!StringUtils.isEmpty(dmpProjectRequest.getProjectDesc())) {
			param.put("projectDesc", dmpProjectRequest.getProjectDesc());
		}
		//发布目标
		if (dmpProjectRequest.getPublishTargetId() != null) {
			param.put("publishTargetId", dmpProjectRequest.getPublishTargetId());
		}
		//启动调度周期
		if (!StringUtils.isEmpty(dmpProjectRequest.getIsEnableScheduler())) {
			param.put("isEnableScheduler", dmpProjectRequest.getIsEnableScheduler());
		}
		//允许在本项目中直接编辑任务和代码
		if (!StringUtils.isEmpty(dmpProjectRequest.getIsEditTaskCode())) {
			param.put("isEditTaskCode", dmpProjectRequest.getIsEditTaskCode());
		}
		//能否下载select结果
		if (!StringUtils.isEmpty(dmpProjectRequest.getIsEnableSelect())) {
			param.put("isEnableSelect", dmpProjectRequest.getIsEnableSelect());
		}
		//数据仓库项目名称
		if (!StringUtils.isEmpty(dmpProjectRequest.getDwName())) {
			param.put("dwName", dmpProjectRequest.getDwName());
		}
		//数据仓库访问身份类型
		if (!StringUtils.isEmpty(dmpProjectRequest.getDwAccessType())) {
			param.put("dwAccessType", dmpProjectRequest.getDwAccessType());
		}
		//使用ACL授权
		if (!StringUtils.isEmpty(dmpProjectRequest.getDwIsEnableAcl())) {
			param.put("dwIsEnableAcl", dmpProjectRequest.getDwIsEnableAcl());
		}
		//允许对象创建者访问对象
		if (!StringUtils.isEmpty(dmpProjectRequest.getDwIsEnableAccessObj())) {
			param.put("dwIsEnableAccessObj", dmpProjectRequest.getDwIsEnableAccessObj());
		}
		//允许对象创建者授权对象
		if (!StringUtils.isEmpty(dmpProjectRequest.getDwIsEnableAuthObj())) {
			param.put("dwIsEnableAuthObj", dmpProjectRequest.getDwIsEnableAuthObj());
		}
		//项目空间数据保护
		if (!StringUtils.isEmpty(dmpProjectRequest.getDwIsProtectData())) {
			param.put("dwIsProtectData", dmpProjectRequest.getDwIsProtectData());
		}
		//数据状态
		if (!StringUtils.isEmpty(dmpProjectRequest.getDataStatus())) {
			param.put("dataStatus", dmpProjectRequest.getDataStatus());
		}
		//创建用户ID
		if (!StringUtils.isEmpty(dmpProjectRequest.getCreateUserId())) {
			param.put("createUserId", dmpProjectRequest.getCreateUserId());
		}
		//数据创建时间起
		if (dmpProjectRequest.getCreateTimeStart() != null) {
			param.put("createTimeStart", dmpProjectRequest.getCreateTimeStart());
		}
		//数据创建时间止
		if (dmpProjectRequest.getCreateTimeEnd() != null) {
			param.put("createTimeEnd", dmpProjectRequest.getCreateTimeEnd());
		}
		//创建用户ID
		if (!StringUtils.isEmpty(dmpProjectRequest.getUpdateUserId())) {
			param.put("updateUserId", dmpProjectRequest.getUpdateUserId());
		}
		//
		if (!StringUtils.isEmpty(dmpProjectRequest.getImpalaJdbcUrl())) {
			param.put("impalaJdbcUrl", dmpProjectRequest.getImpalaJdbcUrl());
		}
		//
		if (!StringUtils.isEmpty(dmpProjectRequest.getImpalaMasterFqdn())) {
			param.put("impalaMasterFqdn", dmpProjectRequest.getImpalaMasterFqdn());
		}
		//数据更新时间起
		if (dmpProjectRequest.getUpdateTimeStart() != null) {
			param.put("updateTimeStart", dmpProjectRequest.getUpdateTimeStart());
		}
		//数据更新时间止
		if (dmpProjectRequest.getUpdateTimeEnd() != null) {
			param.put("updateTimeEnd", dmpProjectRequest.getUpdateTimeEnd());
		}
		
		// 如果没有设置排序，默认时间降序
		if (StringUtils.isEmpty(dmpProjectRequest.getOrderColumn())) {
			dmpProjectRequest.setOrderColumn("createTime");
			dmpProjectRequest.setOrderType("DESC");
		}

		String obderByStr = dmpProjectRequest.takeOrderByStr(DmpProjectRequest.class);
        Page page = null;
        if (obderByStr==null) {
        	page = PageHelper.startPage(dmpProjectRequest.getPageNum(), dmpProjectRequest.getPageSize());
		} else {
			page = PageHelper.startPage(dmpProjectRequest.getPageNum(), dmpProjectRequest.getPageSize(), obderByStr);
		}
		List<DmpProject> dmpProjects = dmpProjectDao.findListByMap(param);
		List<DmpProjectDto> list = MyDmpProjectConverter.INSTANCE().domain2dto(dmpProjects);
		
		PageInfo<DmpProjectDto> pageInfo = new PageInfo<>(list);
		
		pageInfoResponse.setCode(StatuConstant.SUCCESS_CODE);
		pageInfoResponse.setMessage("查询成功");
		pageInfoResponse.setData(pageInfo);
		return pageInfoResponse;
	}
	
	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpProjectService#findList(com.ycxc.vmts.controller.bean.DmpProjectRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public BaseBeanResponse<DmpProjectDto> findList(DmpProjectRequest dmpProjectRequest, HttpServletRequest httpRequest)
			throws Exception {
		BaseBeanResponse<DmpProjectDto> baseBeanResponse = new BaseBeanResponse<>();
		
		Map<String, Object> param = new HashMap<>();
		//ID
		if (dmpProjectRequest.getId() != null) {
			param.put("id", dmpProjectRequest.getId());
		}
		//管理员帐号
		if (!StringUtils.isEmpty(dmpProjectRequest.getOwnerId())) {
			param.put("ownerId", dmpProjectRequest.getOwnerId());
		}
		//生产帐号
		if (!StringUtils.isEmpty(dmpProjectRequest.getProductionId())) {
			param.put("productionId", dmpProjectRequest.getProductionId());
		}
		//数据仓库帐号
		if (!StringUtils.isEmpty(dmpProjectRequest.getDataWarehouseId())) {
			param.put("dataWarehouseId", dmpProjectRequest.getDataWarehouseId());
		}
		//项目名称
		if (!StringUtils.isEmpty(dmpProjectRequest.getName())) {
			param.put("name", dmpProjectRequest.getName());
		}
		//项目显示名称
		if (!StringUtils.isEmpty(dmpProjectRequest.getDisplayName())) {
			param.put("displayName", dmpProjectRequest.getDisplayName());
		}
		//项目描述
		if (!StringUtils.isEmpty(dmpProjectRequest.getProjectDesc())) {
			param.put("projectDesc", dmpProjectRequest.getProjectDesc());
		}
		//发布目标
		if (dmpProjectRequest.getPublishTargetId() != null) {
			param.put("publishTargetId", dmpProjectRequest.getPublishTargetId());
		}
		//启动调度周期
		if (!StringUtils.isEmpty(dmpProjectRequest.getIsEnableScheduler())) {
			param.put("isEnableScheduler", dmpProjectRequest.getIsEnableScheduler());
		}
		//允许在本项目中直接编辑任务和代码
		if (!StringUtils.isEmpty(dmpProjectRequest.getIsEditTaskCode())) {
			param.put("isEditTaskCode", dmpProjectRequest.getIsEditTaskCode());
		}
		//能否下载select结果
		if (!StringUtils.isEmpty(dmpProjectRequest.getIsEnableSelect())) {
			param.put("isEnableSelect", dmpProjectRequest.getIsEnableSelect());
		}
		//数据仓库项目名称
		if (!StringUtils.isEmpty(dmpProjectRequest.getDwName())) {
			param.put("dwName", dmpProjectRequest.getDwName());
		}
		//数据仓库访问身份类型
		if (!StringUtils.isEmpty(dmpProjectRequest.getDwAccessType())) {
			param.put("dwAccessType", dmpProjectRequest.getDwAccessType());
		}
		//使用ACL授权
		if (!StringUtils.isEmpty(dmpProjectRequest.getDwIsEnableAcl())) {
			param.put("dwIsEnableAcl", dmpProjectRequest.getDwIsEnableAcl());
		}
		//允许对象创建者访问对象
		if (!StringUtils.isEmpty(dmpProjectRequest.getDwIsEnableAccessObj())) {
			param.put("dwIsEnableAccessObj", dmpProjectRequest.getDwIsEnableAccessObj());
		}
		//允许对象创建者授权对象
		if (!StringUtils.isEmpty(dmpProjectRequest.getDwIsEnableAuthObj())) {
			param.put("dwIsEnableAuthObj", dmpProjectRequest.getDwIsEnableAuthObj());
		}
		//项目空间数据保护
		if (!StringUtils.isEmpty(dmpProjectRequest.getDwIsProtectData())) {
			param.put("dwIsProtectData", dmpProjectRequest.getDwIsProtectData());
		}
		//数据状态
		if (!StringUtils.isEmpty(dmpProjectRequest.getDataStatus())) {
			param.put("dataStatus", dmpProjectRequest.getDataStatus());
		}
		//创建用户ID
		if (!StringUtils.isEmpty(dmpProjectRequest.getCreateUserId())) {
			param.put("createUserId", dmpProjectRequest.getCreateUserId());
		}
		//数据创建时间起
		if (dmpProjectRequest.getCreateTimeStart() != null) {
			param.put("createTimeStart", dmpProjectRequest.getCreateTimeStart());
		}
		//数据创建时间止
		if (dmpProjectRequest.getCreateTimeEnd() != null) {
			param.put("createTimeEnd", dmpProjectRequest.getCreateTimeEnd());
		}
		//创建用户ID
		if (!StringUtils.isEmpty(dmpProjectRequest.getUpdateUserId())) {
			param.put("updateUserId", dmpProjectRequest.getUpdateUserId());
		}
		//
		if (!StringUtils.isEmpty(dmpProjectRequest.getImpalaJdbcUrl())) {
			param.put("impalaJdbcUrl", dmpProjectRequest.getImpalaJdbcUrl());
		}
		//
		if (!StringUtils.isEmpty(dmpProjectRequest.getImpalaMasterFqdn())) {
			param.put("impalaMasterFqdn", dmpProjectRequest.getImpalaMasterFqdn());
		}
		//数据更新时间起
		if (dmpProjectRequest.getUpdateTimeStart() != null) {
			param.put("updateTimeStart", dmpProjectRequest.getUpdateTimeStart());
		}
		//数据更新时间止
		if (dmpProjectRequest.getUpdateTimeEnd() != null) {
			param.put("updateTimeEnd", dmpProjectRequest.getUpdateTimeEnd());
		}
		
		List<DmpProject> dmpProjects = dmpProjectDao.findListByMap(param);
		List<DmpProjectDto> list = MyDmpProjectConverter.INSTANCE().domain2dto(dmpProjects);
		
		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("查询成功");
		baseBeanResponse.setDatas(list);
		return baseBeanResponse;
	}
	
	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpProjectService#deleteById(com.ycxc.vmts.controller.bean.DmpProjectRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseResponse softDeleteById(Integer id, HttpServletRequest httpRequest)
			throws Exception {
		BaseResponse baseResponse = new BaseResponse();
		
		dmpProjectDao.delete(id);
		
		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("软删除成功");
		
		return baseResponse;
	}
	
	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpProjectService#add(com.ycxc.vmts.entity.DmpProject, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseBeanResponse<DmpProject> add(DmpProject dmpProject, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpProject> baseBeanResponse = new BaseBeanResponse<>();
		
		DmpMember member = SessionUtils.getSecurityUser();
		if (member==null) {
			baseBeanResponse.setCode(StatuConstant.CODE_NOT_SIGNED);
			baseBeanResponse.setMessage("用户未登录");
			return baseBeanResponse;
		}
		//设置拥有者
		dmpProject.setOwnerId(member.getUsername());
		//创建人
		dmpProject.setCreateUserId(member.getUserId().toString());
		//创建时间
		dmpProject.setCreateTime(new Date());
		//默认data_status
		dmpProject.setDataStatus("1");
		
		dmpProjectDao.insert(dmpProject);
		
		//设置projectId
		DmpProjectConfigInfo dmpProjectConfigInfo = dmpProject.getDmpProjectConfigInfo();
		dmpProjectConfigInfo.setProjectId(dmpProject.getId());
		
		//保存项目配置信息
		dmpProjectConfigInfoService.addOrEdit(dmpProjectConfigInfo, httpRequest);
		
		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("新增成功");
		baseBeanResponse.setData(dmpProject);
		
		return baseBeanResponse;
	}

	/**
	 *统计任务运行状况
	 */
	@Override
	public BaseBeanResponse<IndexProjAzkabanStatuDto> projAzkabanStatu(Integer projectId, HttpServletRequest httpRequest)
			throws Exception {
		BaseBeanResponse<IndexProjAzkabanStatuDto> baseBeanResponse = new BaseBeanResponse<IndexProjAzkabanStatuDto>();
		String projectName = "jz_workflow_"+projectId;
		IndexProjAzkabanStatuDto dto = projectFlowsMapper.projAzkabanStatu(projectName);
		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("统计成功");
		baseBeanResponse.setData(dto);
		return baseBeanResponse;
	}
	
	/**
	 *根据projectId获取项目配置信息
	 */
	@Override
	public DmpProjectSystemInfo getProjectSystemInfo(Long projectId)throws Exception{
		DmpProjectSystemInfo info = new DmpProjectSystemInfo();
		List<DmpProjectSystemInfo> list =  dmpProjectDao.getProjectSystemInfo(projectId);
		if(null != list && list.size() > 0) {
			info = list.get(0);
		}
		return info;
	}

	/**
	 *获取登录用户的项目
	 */
	@Override
	public BaseBeanResponse<DmpProjectDto> findListByUserId(HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpProjectDto> baseBeanResponse = new BaseBeanResponse<DmpProjectDto>();
		
		String userId = SessionUtils.getCurrentUserId();
		if (StringUtils.isEmpty(userId)) {
			baseBeanResponse.setCode(StatuConstant.CODE_ERROR_REQUESTMETHOD);
			baseBeanResponse.setMessage("用户未登录！");
			return baseBeanResponse;
		}
		
		List<DmpProjectDto> list = dmpProjectDao.findListByUserId(userId);
		
		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("查询成功");
		baseBeanResponse.setDatas(list);
		return baseBeanResponse;
	}
	
}
